unit main;

{ =====
PX ver 4     copyright (C) 2000 R. Collins   rcoll@rtd.com

This software is provided "as-is".  This software comes without warranty
or guarantee, explicit or implied.  Use this software at your own risk.
The author will not be liable for any damage to equipment, data, or information
that may result while using this software.

By using this software, you agree to the conditions stated above.

This software may be used freely, provided the author's name and copyright
statement remain a part of the source code.

===== }



interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, IniFiles, Menus, ExtCtrls,
  Help, ExtDlgs;

const
    kMaxScreens = 10;                   // max screens vert and horz
    kMaxWindows = 256;                  // number of windows to jugle around
    kSection    = 'PX4';                // section heading for ini file
    kIniName    = 'PX4.INI';            // ini file name

type
    TWinData    = record                // holds info for a single window
        id:       hWnd;
        left:     integer;
        right:    integer;
        top:      integer;
        bottom:   integer;
    end;

    TWinList    = array[1..kMaxWindows] of TWinData;

type
    TBmpList    = array[1..kMaxScreens, 1..kMaxScreens] of string;



type
  TForm1 = class(TForm)
    PopupMenu1: TPopupMenu;
    ColorDialog1: TColorDialog;
    BackgroundColor1: TMenuItem;
    ForegroundColor1: TMenuItem;
    Exit1: TMenuItem;
    HorzCount1: TMenuItem;
    VertCount1: TMenuItem;
    N11: TMenuItem;
    N21: TMenuItem;
    N31: TMenuItem;
    N41: TMenuItem;
    N51: TMenuItem;
    N61: TMenuItem;
    N71: TMenuItem;
    N81: TMenuItem;
    N91: TMenuItem;
    N101: TMenuItem;
    N12: TMenuItem;
    N22: TMenuItem;
    N32: TMenuItem;
    N42: TMenuItem;
    N52: TMenuItem;
    N62: TMenuItem;
    N72: TMenuItem;
    N82: TMenuItem;
    N92: TMenuItem;
    N102: TMenuItem;
    CurrentColor1: TMenuItem;
    Help1: TMenuItem;
    N1: TMenuItem;
    N2: TMenuItem;
    TimerInterval1: TMenuItem;
    N01Second1: TMenuItem;
    N02Second1: TMenuItem;
    N05Second1: TMenuItem;
    N1Second1: TMenuItem;
    N2Second1: TMenuItem;
    N5Second1: TMenuItem;
    N10Second1: TMenuItem;
    NeverUpdate1: TMenuItem;
    Timer1: TTimer;
    Wallpaper1: TMenuItem;
    OpenPictureDialog1: TOpenPictureDialog;

    procedure FormClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FormShow(Sender: TObject);

    procedure BackgroundColor1Click(Sender: TObject);
    procedure CurrentColor1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure ForegroundColor1Click(Sender: TObject);
    procedure Help1Click(Sender: TObject);
    procedure HorzCountClick(Sender: TObject);
    procedure TimerClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure VertCountClick(Sender: TObject);
    procedure Wallpaper1Click(Sender: TObject);

  private
    fIniName:       string;                 // ini file name
    fWriteOK:       boolean;                // can we write to INI file yet?
    fMouseX:        integer;                // where mouse was clicked
    fMouseY:        integer;                // where mouse was clicked
    fBackColor:     TColor;                 // background color
    fForeColor:     TColor;                 // foreground drawing color
    fCurColor:      TColor;                 // current screen background
    fColCount:      integer;                // number of horz screens
    fRowCount:      integer;                // number of vert screens
    fColCurrent:    integer;                // current screen
    fRowCurrent:    integer;                // current screen
    fWinList:       TWinList;               // windows to be moved
    fBmpList:       TBmpList;               // Windows background images


    procedure FormMoved(var msg: TWMMove);  message WM_Move;

    procedure DisplayWallpaper(filename: string);
    procedure GetWindowList;
    procedure ReadIniFile;
    procedure SetScreenCount(horz, vert: integer);
    procedure SetTimerInterval(msec: integer);
    procedure WriteIniFile;



  public
    { Public declarations }
  end;

var
  Form1: TForm1;

{ ---------------------------------------------------------------------------- }
{ ---------------------------------------------------------------------------- }


implementation

{$R *.DFM}

{ ---------------------------------------------------------------------------- }
{ FORM EVENTS ---------------------------------------------------------------- }

{ ---------------------------------------------------------------------------- }
{ this gets called whenever the user clicks the mouse on the main form }
{ this procedure will: }
{   determine the new screen to display from the coords of the mouse click }
{   the new screen is defined by vars "fColCurrent" and "fRowCurrent" }
{   display a new screen background picture if the user defined one }
{   get a list of open top-level windows and adjust their coords for the new screen }
{   the new window position is adjusted as integer steps of physical screen size }

procedure TForm1.FormClick(Sender: TObject);
var
    i,n:        integer;
    pw,ph:      integer;
    nc,nr:      integer;
    hc,vc:      integer;
    hn,vn:      integer;
    l,t,w,h:    integer;
begin

// size of a single pane on the PX display

pw := (ClientRect.Right - ClientRect.Left) div fColCount;
ph := (ClientRect.Bottom - ClientRect.Top) div fRowCount;

// mouse click coords determine which new screen (col & row) to switch to

nc := (fMouseX div pw) + 1;
nr := (fMouseY div ph) + 1;
if (nc < 1) then nc := 1;
if (nc > kMaxScreens) then nc := kMaxScreens;
if (nr < 1) then nr := 1;
if (nr > kMaxScreens) then nr := kMaxScreens;

// do we need to display new wall paper?
// do this before we move a bunch of windows on top of it

if (fBmpList[nc, nr] <> fBmpList[fColCurrent, fRowCurrent]) then
    begin
    DisplayWallpaper(fBmpList[nc, nr]);
    end;

// get list of open top-level windows

GetWindowList;

// compute offsets for current screen and new screen

hc := (fColCurrent - 1) * Screen.Width;
vc := (fRowCurrent - 1) * Screen.Height;
hn := (nc - 1) * Screen.Width;
vn := (nr - 1) * Screen.Height;

// move windows to new coords to match offset from (new) current screen
// do not move our current pager PX window

for i := 1 to kMaxWindows do
if ((fWinList[i].id <> 0) and (fWinList[i].id <> Handle)) then
    begin

// normalize to screen 1,1

    fWinList[i].left   := fWinList[i].left   + hc;
    fWinList[i].top    := fWinList[i].top    + vc;
    fWinList[i].right  := fWinList[i].right  + hc;
    fWinList[i].bottom := fWinList[i].bottom + vc;

// and then to our new screen

    fWinList[i].left   := fWinList[i].left   - hn;
    fWinList[i].top    := fWinList[i].top    - vn;
    fWinList[i].right  := fWinList[i].right  - hn;
    fWinList[i].bottom := fWinList[i].bottom - vn;

// copy out rectangle coords

    l :=  fWinList[i].left;
    t :=  fWinList[i].top;
    w := (fWinList[i].right  - fWinList[i].left);
    h := (fWinList[i].bottom - fWinList[i].top);

// and move the window

    MoveWindow(fWinList[i].id, l, t, w, h, true);
    end;

// keep track of new page

fColCurrent := nc;
fRowCurrent := nr;

// make sure the form display is updated

Invalidate;
end;

{ ---------------------------------------------------------------------------- }
{ the only thing to do when finished is make absolutely sure we saved everything }

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
WriteIniFile;
end;

{ ---------------------------------------------------------------------------- }
{ this is where it all gets started }
{ a key part of this program is the INI file used to store saved and default }
{ window settings.  I hate the Windows registry.  One trivial mistake can }
{ prevent your machine from booting.  A major mistake means you start looking }
{ for a new career.  So, the INI file is resurrected.  It may be old-fashioned, }
{ but it works, by gum.  The INI file for PX is stored in the directory where }
{ PX is started from; the idea is that you can de-install PX just by deleteing }
{ the entire PX directory. No muss, no fuss.}

procedure TForm1.FormCreate(Sender: TObject);
var
    i,n:    integer;
    l,t:    integer;
    s:      string;
begin

// do not attempt to write INI file for a while
// there is no need to save a bunch of incomplete parameters;
// changing Left,Top or Width,Height will cause a call to WriteIniFile
// before we are ready; just save it all till later

fWriteOK := false;

// ini file name

fIniName := ExtractFilePath(application.ExeName);
i := length(fIniName);
if ((i = 0) or (fIniName[i] <> '\')) then fIniName := fIniName + '\';
fIniName := fIniName + kIniName;

// read in defaults

ReadIniFile;

// no mouse clicks yet

fMouseX := 0;
fMouseY := 0;

// current screen

fColCurrent := 1;
fRowCurrent := 1;

// get list of current windows

GetWindowList;

// if PX died earlier and this is a restart, we may not actually
// be starting on screen 1,1

l := 0;    // left edge
t := 0;    // top edge
i := 0;
while (i < kMaxWindows) do
    begin
    i := i + 1;
    if (fWinList[i].right < l) then
        begin
        l := l - Screen.Width;
        fColCurrent := fColCurrent + 1;
        i := 0;
        end
    else if (fWinList[i].bottom < t) then
        begin
        t := t - Screen.Height;
        fRowCurrent := fRowCurrent + 1;
        i := 0;
        end;
    end;

if (fColCurrent > kMaxScreens) then fColCurrent := kMaxScreens;
if (fRowCurrent > kMaxScreens) then fRowCurrent := kMaxScreens;
if (fColCurrent > fColCount)   then fColCount   := fColCurrent;
if (fRowCurrent > fRowCount)   then fRowCount   := fRowCurrent;

// our own window must always be on the current screen

while ((Left + Width) < 0)             do Left := Left + Screen.Width;
while (Left           > Screen.Width)  do Left := Left - Screen.Width;
while ((Top + Height) < 0)             do Top  := Top  + Screen.Height;
while (Top            > Screen.Height) do Top  := Top  - Screen.Height;

// now we can write a new INI file when we need to

fWriteOK := true;
end;

{ ---------------------------------------------------------------------------- }
{ this gets called by the OS whenever the main form needs to be re-drawn }
{ this relies on GetWindowList having already been called and the fWindowList }
{ array already filled with valid data }

procedure TForm1.FormPaint(Sender: TObject);
var
    i,n:    integer;
    x,y:    integer;
    w,h:    integer;
    pw,ph:  integer;
    ho,vo:  integer;
    rr:     TRect;
begin

// fill in the background color

Canvas.Brush.Style := bsSolid;
Canvas.Brush.Color := fBackColor;
Canvas.FillRect(ClientRect);

// size of each screen pane on the canvas

pw := (ClientRect.Right - ClientRect.Left) div fColCount;
ph := (ClientRect.Bottom - ClientRect.Top) div fRowCount;
if (pw < 1) then pw := 1;
if (ph < 1) then ph := 1;

// background of the current screen

Canvas.Brush.Color := fCurColor;
x := fColCurrent;
y := fRowCurrent;
rr.Left   := (x - 1) * pw;
rr.Top    := (y - 1) * ph;
rr.Right  := rr.Left + pw;
rr.Bottom := rr.Top  + ph;
if (x = fColCount) then rr.Right  := ClientRect.Right;
if (y = fRowCount) then rr.Bottom := ClientRect.Bottom;
Canvas.FillRect(rr);

// draw the screen panes; just a bunch of lines that make rectangles

Canvas.Brush.Color := fForeColor;

for y := 1 to fRowCount do
for x := 1 to fColCount do
    begin
    rr.Left   := (x - 1) * pw;
    rr.Top    := (y - 1) * ph;
    rr.Right  := rr.Left + pw;
    rr.Bottom := rr.Top  + ph;
    if (x = fColCount) then rr.Right  := ClientRect.Right;
    if (y = fRowCount) then rr.Bottom := ClientRect.Bottom;
    Canvas.FrameRect(rr);
    end;

// compute offsets for current screen
// this how far we had to move a window to make it appear to be on
// the (imaginary) screen off to one side or the other

ho := (fColCurrent - 1) * Screen.Width;
vo := (fRowCurrent - 1) * Screen.Height;

// fill in the window frames

Canvas.Brush.Color := fForeColor;

for i := 1 to kMaxWindows do
if (fWinList[i].id <> 0) then
    begin

// copy out window rectangle coords

    rr.Left   := fWinList[i].left;
    rr.Right  := fWinList[i].right;
    rr.Top    := fWinList[i].top;
    rr.Bottom := fWinList[i].bottom;

// normalize to page 1,1

    rr.Left   := rr.Left   + ho;
    rr.Right  := rr.Right  + ho;
    rr.Top    := rr.Top    + vo;
    rr.Bottom := rr.Bottom + vo;

// scale to fit form canvas

    rr.Left   := (longint(rr.Left)   * longint(pw)) div longint(Screen.Width);
    rr.Right  := (longint(rr.Right)  * longint(pw)) div longint(Screen.Width);
    rr.Top    := (longint(rr.Top)    * longint(ph)) div longint(Screen.Height);
    rr.Bottom := (longint(rr.Bottom) * longint(ph)) div longint(Screen.Height);

// draw the frame

    Canvas.FrameRect(rr);
    end;
end;

{ ---------------------------------------------------------------------------- }
{ if the user resizes the main form, save the new size }

procedure TForm1.FormResize(Sender: TObject);
begin
WriteIniFile;
Invalidate;
end;


{ ---------------------------------------------------------------------------- }
{ save the coords for the beginning of a mouse click }

procedure TForm1.FormMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
fMouseX := X;
fMouseY := Y;
end;

{ ---------------------------------------------------------------------------- }

procedure TForm1.FormShow(Sender: TObject);
var
    h:      boolean;
begin

// if the INI file does not exist, then this is probably the first
// time PX has been run.  by default, open the help file on the first run

if (not FileExists(fIniName)) then HelpDialog.Visible := true;
end;

{ ---------------------------------------------------------------------------- }
{ MENU CLICK EVENTS ---------------------------------------------------------- }

{ ---------------------------------------------------------------------------- }
{ change default background color of the main form }

procedure TForm1.BackgroundColor1Click(Sender: TObject);
begin
if (ColorDialog1.Execute) then fBackColor := ColorDialog1.Color;
Color := fBackColor;
WriteIniFile;
Invalidate;
end;

{ ---------------------------------------------------------------------------- }

procedure TForm1.CurrentColor1Click(Sender: TObject);
begin
if (ColorDialog1.Execute) then fCurColor := ColorDialog1.Color;
WriteIniFile;
Invalidate;
end;

{ ---------------------------------------------------------------------------- }
{ when the user wants to leave }

procedure TForm1.Exit1Click(Sender: TObject);
begin
WriteIniFile;
Halt;
end;


{ ---------------------------------------------------------------------------- }
{ change color of the lines that define screen panes and window edges }

procedure TForm1.ForegroundColor1Click(Sender: TObject);
begin
if (ColorDialog1.Execute) then fForeColor := ColorDialog1.Color;
WriteIniFile;
Invalidate;
end;


{ ---------------------------------------------------------------------------- }
{ display the help file }

procedure TForm1.Help1Click(Sender: TObject);
begin
HelpDialog.Visible := true;
end;

{ ---------------------------------------------------------------------------- }
{ change number of horizontal screen panes }

procedure TForm1.HorzCountClick(Sender: TObject);
var
    i,n:    integer;
begin
if (Sender is TMenuItem) then
    begin
    n := TMenuItem(Sender).Tag;     // by happy coincidence, numbered 1..10
    SetScreenCount(n, fRowCount);
    WriteIniFile;
    end;
end;

{ ---------------------------------------------------------------------------- }

procedure TForm1.TimerClick(Sender: TObject);
var
    i,n:    integer;
begin
if (Sender is TMenuItem) then
    begin
    n := TMenuItem(Sender).Tag;     // tag continus number of milliseconds
    SetTimerInterval(n);
    WriteIniFile;
    end;
end;

{ ---------------------------------------------------------------------------- }
{ called automagically by the timer every 'n' milliseconds }
{ it just gets an updated list of open windows, and causes the main form }
{ to be re-painted }

procedure TForm1.Timer1Timer(Sender: TObject);
begin
GetWindowList;
Invalidate;
end;


{ ---------------------------------------------------------------------------- }
{ change the number of vertical screens defined }

procedure TForm1.VertCountClick(Sender: TObject);
var
    i,n:    integer;
begin
if (Sender is TMenuItem) then
    begin
    n := TMenuItem(Sender).Tag;     // a value from 1..10
    SetScreenCount(fColCount, n);
    WriteIniFile;
    end;
end;


{ ---------------------------------------------------------------------------- }
{ the user selects a new bitmap file for the wallpaper on this screen }

procedure TForm1.Wallpaper1Click(Sender: TObject);
var
    i,n:    integer;
    s:      string;
begin
if (OpenPictureDialog1.Execute) then
    begin
    fBmpList[fColCurrent, fRowCurrent] := OpenPictureDialog1.Filename;
    DisplayWallpaper(fBmpList[fColCurrent, fRowCurrent]);
    WriteIniFile;
    end;
end;

{ ---------------------------------------------------------------------------- }
{ WINDOWS MESSAGES ----------------------------------------------------------- }

{ ---------------------------------------------------------------------------- }
{ this is called by Windows when the user moves the form on the screen }
{ save the new Left, Top coords of the main form }
{ surprisingly, this is not a Delphi event; e.g., OnMove }

procedure TForm1.FormMoved(var msg: TWMMove);
begin
WriteIniFile;
end;


{ ---------------------------------------------------------------------------- }
{ UTILITY ROUTINES ----------------------------------------------------------- }


{ ---------------------------------------------------------------------------- }
{ display new wall paper; catch and ignore errors }

procedure TForm1.DisplayWallpaper(filename: string);
var
    i,n:    integer;
begin
try
    if ((length(filename) > 0) and (FileExists(filename))) then
        SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, @filename[1], 0);
except
    i := 0;     // a no-op to fill in the block
end;    // try..except
end;


{ ---------------------------------------------------------------------------- }
{ load new list of active windows }

procedure TForm1.GetWindowList;
var
    i,n:    integer;
    rw:     hWnd;
    ww:     hWnd;
    ra:     TRect;
begin

// start with the root window

rw := GetDesktopWindow;
GetWindowRect(rw, ra);

n := 1;

fWinList[n].id     := rw;
fWinList[n].left   := ra.left;
fWinList[n].right  := ra.right;
fWinList[n].top    := ra.top;
fWinList[n].bottom := ra.bottom;

// then with first child under the root

ww := GetTopWindow(rw);

// and add it to list

while ((ww <> 0) and (n < kMaxWindows)) do
    begin

// get basic window info

    GetWindowRect(ww, ra);

// valid window? ignore icons and windows too small to see

    if ((IsWindow(ww)) and (not IsIconic(ww)) and (IsWindowVisible(ww)) and
        (ra.right > ra.left) and (ra.bottom > ra.top)) then
        begin
        n := n + 1;
        fWinList[n].id     := ww;
        fWinList[n].left   := ra.left;
        fWinList[n].right  := ra.right;
        fWinList[n].top    := ra.top;
        fWinList[n].bottom := ra.bottom;
        end;

{ get next window in Z-order under root window }

    ww := GetNextWindow(ww, GW_HWNDNEXT);
    end;

// empty the rest of the window records

while (n < kMaxWindows) do
    begin
    n := n + 1;
    fWinList[n].id := 0;
    end;
end;


{ ---------------------------------------------------------------------------- }
{ read default values from the INI file }

procedure TForm1.ReadIniFile;
var
    i,n:    integer;
    l,t:    integer;
    w,h:    integer;
    s:      string;
    ff:     TIniFile;
begin

// open the ini file

ff := TIniFile.Create(fIniName);

// form position and size

l := ff.ReadInteger(kSection, 'LEFT',    5);
t := ff.ReadInteger(kSection, 'TOP',     5);
w := ff.ReadInteger(kSection, 'WIDTH', 300);
h := ff.ReadInteger(kSection, 'HEIGHT', 80);

SetBounds(l, t, w, h);

// colors

fBackColor := ff.ReadInteger(kSection, 'BACKGROUND', clNavy);
fForeColor := ff.ReadInteger(kSection, 'FOREGROUND', clWhite);
fCurColor  := ff.ReadInteger(kSection, 'CURCOLOR',   clBlue);
Color := fBackColor;

// number of screens to display

fColCount := ff.ReadInteger(kSection, 'COLUMNS', 4);
fRowCount := ff.ReadInteger(kSection, 'ROWS',    1);
SetScreenCount(fColCount, fRowCount);

// timer interval

n := ff.ReadInteger(kSection, 'TIMER', 1000);
SetTimerInterval(n);

// bitmaps for screens

for i := 1 to kMaxScreens do    // cols
for n := 1 to kMaxScreens do    // rows
    begin
    s := 'BMP_' + IntToStr(i) + '_' + IntToStr(n);
    fBmpList[i,n] := ff.ReadString(kSection, s, '');
    end;

// done

ff.Free;
end;



{ ---------------------------------------------------------------------------- }
{ set number of horz and vert screens }

procedure TForm1.SetScreenCount(horz, vert: integer);
begin
if ((1 <= horz) and (horz <= kMaxScreens)) then
    begin
    fColCount := horz;

    N11.Checked  := false;
    N21.Checked  := false;
    N31.Checked  := false;
    N41.Checked  := false;
    N51.Checked  := false;
    N61.Checked  := false;
    N71.Checked  := false;
    N81.Checked  := false;
    N91.Checked  := false;
    N101.Checked := false;

    if      (horz =  1) then N11.Checked  := true
    else if (horz =  2) then N21.Checked  := true
    else if (horz =  3) then N31.Checked  := true
    else if (horz =  4) then N41.Checked  := true
    else if (horz =  5) then N51.Checked  := true
    else if (horz =  6) then N61.Checked  := true
    else if (horz =  7) then N71.Checked  := true
    else if (horz =  8) then N81.Checked  := true
    else if (horz =  9) then N91.Checked  := true
    else if (horz = 10) then N101.Checked := true;
    end;

if ((1 <= vert) and(vert <= 10)) then
    begin
    fRowCount := vert;

    N12.Checked  := false;
    N22.Checked  := false;
    N32.Checked  := false;
    N42.Checked  := false;
    N52.Checked  := false;
    N62.Checked  := false;
    N72.Checked  := false;
    N82.Checked  := false;
    N92.Checked  := false;
    N102.Checked := false;

    if      (vert =  1) then N12.Checked  := true
    else if (vert =  2) then N22.Checked  := true
    else if (vert =  3) then N32.Checked  := true
    else if (vert =  4) then N42.Checked  := true
    else if (vert =  5) then N52.Checked  := true
    else if (vert =  6) then N62.Checked  := true
    else if (vert =  7) then N72.Checked  := true
    else if (vert =  8) then N82.Checked  := true
    else if (vert =  9) then N92.Checked  := true
    else if (vert = 10) then N102.Checked := true;
    end;

Invalidate;
end;



{ ---------------------------------------------------------------------------- }
{ set timer update interval }

procedure TForm1.SetTimerInterval(msec: integer);
begin
N01Second1.Checked   := false;
N02Second1.Checked   := false;
N05Second1.Checked   := false;
N1Second1.Checked    := false;
N2Second1.Checked    := false;
N5Second1.Checked    := false;
N10Second1.Checked   := false;
NeverUpdate1.Checked := false;

if      (msec =   100) then N01Second1.Checked   := true
else if (msec =   200) then N02Second1.Checked   := true
else if (msec =   500) then N05Second1.Checked   := true
else if (msec =  1000) then N1Second1.Checked    := true
else if (msec =  2000) then N2Second1.Checked    := true
else if (msec =  5000) then N5Second1.Checked    := true
else if (msec = 10000) then N10Second1.Checked   := true
else if (msec =     0) then NeverUpdate1.Checked := true;

Timer1.Interval := msec;
Timer1.Enabled := (msec > 0);
end;


{ ---------------------------------------------------------------------------- }
{ save defaults to INI file }

procedure TForm1.WriteIniFile;
var
    i,n:    integer;
    l,t:    integer;
    w,h:    integer;
    s:      string;
    ff:     TIniFile;
begin

// can we write a new INI file?

if ((not fWriteOK) or (length(fIniName) = 0)) then exit;

// open the ini file

ff := TIniFile.Create(fIniName);

// form position and size

ff.WriteInteger(kSection, 'LEFT',   Left);
ff.WriteInteger(kSection, 'TOP',    Top);
ff.WriteInteger(kSection, 'WIDTH',  Width);
ff.WriteInteger(kSection, 'HEIGHT', Height);

// colors

ff.WriteInteger(kSection, 'BACKGROUND', fBackColor);
ff.WriteInteger(kSection, 'FOREGROUND', fForeColor);
ff.WriteInteger(kSection, 'CURCOLOR',   fCurColor);

// number of screens to display

ff.WriteInteger(kSection, 'COLUMNS', fColCount);
ff.WriteInteger(kSection, 'ROWS', fRowCount);

// timer interval

ff.WriteInteger(kSection, 'TIMER', Timer1.Interval);

// bitmaps for screens

for i := 1 to fColCount do      // cols
for n := 1 to fRowCount do      // rows
    begin
    s := 'BMP_' + IntToStr(i) + '_' + IntToStr(n);
    ff.WriteString(kSection, s, fBmpList[i,n]);
    end;

// done

ff.Free;
end;



end.
